<?php

namespace OptimizePress\Integrations\Services\Integrations;

use Exception;
use OPDashboard\Services\GeoIp;
use GuzzleHttp\Client as HttpClient;
use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\Collections\IntegrationListsCollection;
use OptimizePress\Integrations\Collections\IntegrationTagsCollection;
use OptimizePress\Integrations\Integration;
use OptimizePress\Integrations\IntegrationTag;
use OptimizePress\Integrations\IntegrationList;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinErrorResult;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\OptinSuccessResult;
use OptimizePress\Integrations\Services\Contracts\BaseIntegration;
use OptimizePress\Integrations\Services\Contracts\IntegrationServiceInterface;
use OptimizePress\Integrations\Services\Contracts\IntegrationHasTagsInterface;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Support\Log\Log;

/**
 * Integration with EverWebinar
 */
class LeadLoversIntegration extends BaseIntegration implements IntegrationServiceInterface, IntegrationHasTagsInterface
{
    /**
     * Init dependencies and service
     *
     * @param Integration $integration
     */
    public function __construct(Integration $integration)
    {
        // Initialize and setup
        $this->init($integration);

        // Setup base config
        $this->config = [
            'query' => [
                // https://app.leadlovers.com/settings#?tab=profile
                'token' => $integration->getConnectionValue('token'),
            ],
            'headers' => [
                // http://llapi.leadlovers.com/#section/Introduction/Authentication
                'Authorization' => 'Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1bmlxdWVfbmFtZSI6IldlYkFwaSIsInN1YiI6IldlYkFwaSIsInJvbGUiOlsicmVhZCIsIndyaXRlIl0sImlzcyI6Imh0dHA6Ly93ZWJhcGlsbC5henVyZXdlYnNpdGVzLm5ldCIsImF1ZCI6IjFhOTE4YzA3NmE1YjQwN2Q5MmJkMjQ0YTUyYjZmYjc0IiwiZXhwIjoxNjA1NDQxMzM4LCJuYmYiOjE0NzU4NDEzMzh9.YIIpOycEAVr_xrJPLlEgZ4628pLt8hvWTCtjqPTaWMs',
                'Content-Type' => 'application/json',
            ],
        ];

        // HTTP clients and helpers
        $this->client = new HttpClient($this->config);
    }

    /**
     * Check if integration is ready for access
     *
     * @return bool
     */
    public function isReady()
    {
        if ($this->client) {
            return true;
        }

        return false;
    }

    /**
     * Ping the service API
     *
     * @return bool
     */
    public function ping()
    {
        // http://llapi.leadlovers.com/#operation/User_GetUser
        $url = $this->integration->serviceUrl() . '/User';

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->UserEmail)) {
                return true;
            }
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Ping failed. " . $e->getMessage());
        }

        return false;
    }

    /**
     * Return all lists
     *
     * @return IntegrationListsCollection
     */
    public function getLists()
    {
        // http://llapi.leadlovers.com/#operation/Machines_Get
        $url = $this->integration->serviceUrl() . '/Machines';
        $lists = new IntegrationListsCollection;

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->Items)) {
                foreach ($results->Items as $list) {
                    $list = $this->listObject([
                        'id'   => $list->MachineCode,
                        'name' => $list->MachineName,
                    ]);
                    $lists->push($list);
                }

                return $lists;
            }

            Log::error('[LEADLOVERS] Failed to get machine.');
        } catch (Exception $e) {
            Log::error('[LEADLOVERS] Error when fetching machines from service. ' . $e->getMessage());
        }

        return $lists;
    }

    /**
     * Return a list
     *
     * @param  string  $listId
     * @return IntegrationList
     */
    public function getList($listId)
    {
        if (! $listId) {
            return null;
        }

        $groups = $this->getLists();

        if ($groups) {
            foreach ($groups as $group) {
                if ($group->id == $listId) {
                    $list = $group;
                }
            }
        }

        return isset($list) ? $list : null;
    }

    /**
     * Get user fields.
     * Fields are hardcoded based on api parameters
     * when registring user to mashine.
     * http://llapi.leadlovers.com/#operation/Lead_Post
     *
     * @param  string  $listId
     * @return IntegrationFieldsCollection
     */
    public function getFields($listId = null)
    {
        $fields = new IntegrationFieldsCollection;

        if (! $listId) {
            return $fields;
        }

        // Name
        $fields->push(new IntegrationField([ 'id' => 'name', 'name' => 'name', 'label' => 'Name', 'optin_id' => 'Name', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // Email
        $fields->push(new IntegrationField([ 'id' => 'email', 'name' => 'email', 'label' => 'E-Mail', 'optin_id' => 'Email', 'enabled' => true, 'required' => true, 'order' => 0, 'type' => 'email', ]));

        // Phone
        $fields->push(new IntegrationField([ 'id' => 'phone', 'name' => 'phone', 'label' => 'Phone', 'optin_id' => 'Phone', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // Birthday
        $fields->push(new IntegrationField([ 'id' => 'birthday', 'name' => 'birthday', 'label' => 'Birthday', 'optin_id' => 'Birthday', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'date', ]));

        // City
        $fields->push(new IntegrationField([ 'id' => 'city', 'name' => 'city', 'label' => 'City', 'optin_id' => 'City', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // State
        $fields->push(new IntegrationField([ 'id' => 'state', 'name' => 'state', 'label' => 'State', 'optin_id' => 'State', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        // Company
        $fields->push(new IntegrationField([ 'id' => 'company', 'name' => 'company', 'label' => 'Company', 'optin_id' => 'Company', 'enabled' => true, 'required' => false, 'order' => 0, 'type' => 'text', ]));

        return $fields;
    }

    /**
     * Get all tags
     *
     * @return IntegrationTagsCollection
     */
    public function getTags()
    {
        $tags = new IntegrationTagsCollection;
        // http://llapi.leadlovers.com/#tag/Tags
        $url = $this->integration->serviceUrl() . '/Tags';

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->Tags)) {
                foreach ($result->Tags as $tag) {
                    $tags->push(new IntegrationTag([
                        // Title is needed when assigning tag to user
                        'id'   => $tag->Title,
                        'name' => $tag->Title,
                    ]));
                }

                return $tags;
            }

            Log::error("[LEADLOVERS] Failed to get tags.");
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Error when fetching tags from service. ". $e->getMessage());
        }

        return $tags;
    }

    /**
     * Return a specific tag
     *
     * @param  mixed  $tagId
     * @return mixed
     */
    public function getTag($tagId)
    {
        if (! $tagId) {
            return null;
        }

        $allTags = $this->getTags();

        foreach ($allTags as $tag) {
            if ($tag->id === $tagId) {
                return $tag;
            }
        }

        return null;
    }

    /**
     * Get integration email sequences
     *
     * @param string $listId
     * @return mixed
     */
    public function getEmailSequences($listId = null)
    {
        $url = $this->integration->serviceUrl() . '/EmailSequences';
        $this->config['query']['machineCode'] = $listId;

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->Items)) {
                return $result->Items;
            }

            Log::error("[LEADLOVERS] Failed to get email sequence.");
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Error when fetching email sequence from service. ". $e->getMessage());
        }

        return null;
    }

    /**
     * Get sequece level code
     *
     * @param  string  $mashineCode
     * @param  string  $emailSequenceCode
     * @return mixed
     */
    public function getSequenceLevels($machineCode = null, $sequenceCode = null)
    {
        // http://llapi.leadlovers.com/#operation/Levels_Get
        $url = $this->integration->serviceUrl() . '/Levels';
        $this->config['query']['machineCode'] = $machineCode;
        $this->config['query']['sequenceCode'] = $sequenceCode;

        try {
            $response = $this->client->get($url, $this->config);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->Items)) {
                return $result->Items;
            }

            Log::error("[LEADLOVERS] Failed to get sequence level code.");
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Error when fetching sequence level code from service. ". $e->getMessage());
        }

        return null;
    }

    /**
     * Optin to email service
     *
     * @param  OptinData   $optinData
     * @param  LeadOptions $leadOptions
     * @return OptinResult
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null)
    {
        $result = $this->optinErrorResult();
        $listId = $optinData->listId();
        $fields = $this->getFields($listId);
        $data = $this->config;

        // Prepare fields
        foreach ($fields as $field) {
            if ($value = $optinData->data($field->id)) {
                $data['json'][$field->optin_id] = $value;
            }
        };

        $data['json']['MachineCode'] = $listId;
        $data['json']['EmailSequenceCode'] = $optinData->data('optin-email-sequence');
        $data['json']['SequenceLevelCode'] = $optinData->data('optin-sequence-level');

        // Prepare GDPR Note
        $gdprNote = $this->getGdprNote($optinData);
        if (! empty($gdprNote)) {
            $data['json']['Notes'] = $gdprNote;
        }

        // User location
        if (class_exists('\OPDashboard\Services\GeoIp')) {
            $optinData->isFromEu = GeoIp::isFromEu();
        } else {
            $optinData->isFromEu = true;
        }

        // GDPR Tags
        $gdpr = $this->processGdprTags($optinData, true);
        $gdprTags = [];
        if (! empty($gdpr)) {
            foreach ($gdpr as $gdprTag) {
                $gdprTags[] = $gdprTag['id'];
            }
        }

        // Also add default tags
        $optinTags = $optinData->tags();
        $realTags = [];
        if (!empty($optinTags)) {
            foreach ($optinTags as $tag) {
                $realTags[] = $tag;
            }
        }

        $allTags = array_merge($gdprTags, $realTags);
        $data['json']['Tags'] = $allTags;

        $lead = $this->updateLead($data);
        if ($lead) {
            $result = $this->optinSuccessResult();
        }

        return $result;
    }

    /**
     * Register new lead to machine (list)
     *
     * @param array $data
     * @return Boolean
     * @throws OptinException
     */
    public function registerNewLead($data)
    {
        // http://llapi.leadlovers.com/#operation/Lead_Post
        $url = $this->integration->serviceUrl() . '/Lead';
        $listId = $data['json']['MachineCode'];

        try {
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->StatusCode)) {
                return true;
            }

            Log::error("[LEADLOVERS] Failed to subscribe user to machine [$listId].");
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Error when subscribing to machine [$listId]. " . $e->getMessage());
            $this->throwOptinException($e);
        }

        return false;
    }

    /**
     * Update or create registered lead with new data
     *
     * @param array $data
     * @return Boolean
     * @throws OptinException
     */
    public function updateLead($data)
    {
        // http://llapi.leadlovers.com/#operation/Lead_Upsert
        $url = $this->integration->serviceUrl() . '/Lead';
        $listId = $data['json']['MachineCode'];

        try {
            $response = $this->client->put($url, $data);

            if ($response->getStatusCode() === 200 and $results = @json_decode($response->getBody()) and isset($results->Email)) {
                return true;
            }

            Log::error("[LEADLOVERS] Failed to update user to machine [$listId].");
        } catch (Exception $e) {
            Log::error("[LEADLOVERS] Error when subscribing to machine [$listId]. " . $e->getMessage());
            $this->throwOptinException($e);
        }

        return false;
    }
}
