<?php

namespace OptimizePress\Integrations\Services\Integrations;

use GuzzleHttp\Client as HttpClient;
use OPDashboard\Services\GeoIp;
use OptimizePress\Integrations\Services\Contracts\BaseIntegration;
use OptimizePress\Integrations\Services\Contracts\IntegrationServiceInterface;
use OptimizePress\Integrations\Services\Contracts\IntegrationHasTagsInterface;
use OptimizePress\Integrations\Services\Contracts\IntegrationServiceOAuthInterface;
use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\Collections\IntegrationListsCollection;
use OptimizePress\Integrations\Collections\IntegrationTagsCollection;
use OptimizePress\Integrations\Integration;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\IntegrationTag;
use OptimizePress\Integrations\IntegrationList;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinResult;
use Exception;
use OptimizePress\Integrations\Services\Exceptions\AlreadySubscribedException;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use function OptimizePress\Support\array_get;
use OptimizePress\Support\Log\Log;

/**
 * Integration with Emma
 */
class MauticIntegration extends BaseIntegration implements IntegrationServiceInterface, IntegrationServiceOAuthInterface, IntegrationHasTagsInterface
{
    /**
     * Init dependencies and service
     *
     * @param Integration $integration
     */
    public function __construct(Integration $integration)
    {
        // Initialize and setup
        $this->init($integration);

        // Get the client ID and secret
        $clientId = $this->integration->getConnectionValue('client_id')     ?: $this->getEnvClientId();
        $clientSecret = $this->integration->getConnectionValue('client_secret') ?: $this->getEnvClientSecret();
        $mauticUrl = $this->integration->getConnectionValue('url');
        $defaultConfig = $this->integration->config->get('connection');

        // Setup base config
        $this->config = [
            'client_id'         => $clientId,
            'client_secret'     => $clientSecret,
            'api_url'           => $mauticUrl . '/api',
            'authorize_uri'     => $mauticUrl . $defaultConfig['authorization_url'],
            'access_token_uri'  => $mauticUrl . $defaultConfig['access_token_uri'],
            'refresh_token_uri' => $mauticUrl . $defaultConfig['access_token_uri'],
            'redirect_uri'      => $this->integration->getCallbackUrl(),
        ];

        // HTTP clients and helpers
        $this->client = new HttpClient();

        // Set the token if available for integration
        $this->refreshToken();
    }

    /**
     * Authorize the API
     *
     * @return mixed
     */
    public function initAuthorization()
    {
        $authorizeUrl = $this->getAuthorizationUrl();

        // Redirect user to authorization URL
        header("HTTP/1.1 200 OK");
        header("Location: $authorizeUrl");
        exit();
    }

    /**
     * Return URL for authorizing OAuth requests
     *
     * @return string
     */
    public function getAuthorizationUrl()
    {
        // Build up the authorization URL
        $url   = $this->config['authorize_uri'];
        $query = [
            'redirect_uri'  => $this->config['redirect_uri'],
            'client_id'     => $this->config['client_id'],
            'response_type' => 'code',
        ];
        $url = $url . '?' . http_build_query($query);

        return $url;
    }

    /**
     * Return token for authorizing OAuth requests
     *
     * @param  array $requestData
     * @return mixed
     */
    public function getAuthorizationTokenAndSecret(array $requestData)
    {
        $code = isset($requestData['code']) ? $requestData['code'] : null;

        // Prepare request data
        $data = [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->config['client_id'] . ':' . $this->config['client_secret']),
                'Content-Type'  => 'application/x-www-form-urlencoded',
                'Accept'        => 'application/json',
            ],
            'form_params' => [
                'grant_type'    => 'authorization_code',
                'client_id'     => $this->config['client_id'],
                'code'          => $code,
                'redirect_uri'  => $this->config['redirect_uri'],
            ]
        ];

        // And try to make the request
        try {
            $response = $this->client->post($this->config['access_token_uri'], $data);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->access_token)) {
                Log::debug('[MAUTIC] Access token fetched: ' . $result->access_token);

                // Set the token and date
                $this->integration->connection->token = $result->access_token;
                $expiresAt = date('Y-m-d H:i:s', time() + $result->expires_in);

                // And return aggregated result
                return [
                    'token'            => $result->access_token,
                    'token_secret'     => true,
                    'refresh_token'    => $result->refresh_token,
                    'token_expires_at' => $expiresAt,
                ];
            }

            Log::error('[MAUTIC] No access token found in response. ' . @json_decode($response->getBody()));
        } catch (Exception $e) {
            Log::error('[MAUTIC] Error when fetching access token. ' . $e->getMessage());
        }
    }

    /**
     * Since the token never expires we simply return the current data
     *
     * @param  bool  $force
     * @return mixed
     */
    public function refreshToken($force = false)
    {
        // Refresh the toke only if refresh token is set and we're forcing a refresh or the token has expired
        if ($this->getRefreshToken() and $this->isTokenExpired()) {
            Log::debug('[MAUTIC] Refreshing access token for integration "' . $this->integration->provider . '" [' . $this->integration->id . '].');

            // Prepare request data
            $data = [
                'headers' => [
                    'Authorization' => 'Basic ' . base64_encode($this->config['client_id'] . ':' . $this->config['client_secret']),
                    'Content-Type'  => 'application/x-www-form-urlencoded',
                    'Accept'        => 'application/json',
                ],
                'form_params' => [
                    'grant_type'    => 'refresh_token',
                    'refresh_token' => $this->getRefreshToken(),
                ]
            ];
            Log::debug('[MAUTIC] Refresh data: ' . @json_encode($data));

            // And try to make the request
            try {
                $response = $this->client->post($this->config['refresh_token_uri'], $data);

                if ($response->getStatusCode() == 200 and $result = @json_decode($response->getBody()) and isset($result->access_token)) {
                    // Set the token and date
                    $this->integration->connection->token = $result->access_token;
                    $expiresAt = date('Y-m-d H:i:s', time() + $result->expires_in);

                    // Compile aggregated result
                    $tokenData = [
                        'token'            => $result->access_token,
                        'token_secret'     => true,
                        'refresh_token'    => $result->refresh_token,
                        'token_expires_at' => $expiresAt,
                    ];

                    Log::debug('[MAUTIC] Access token refreshed: ' . @json_encode($tokenData));
                    $this->integration->storage()->updateConnectionFields($this->integration->uid, $tokenData);

                    return $tokenData;
                }

                Log::error('[MAUTIC] No access token found in response when refreshing. ' . @json_decode($response->getBody()));
            } catch (Exception $e) {
                Log::error('[MAUTIC] Error when refreshing access token. ' . $e->getMessage());
            }
        }
    }

    /**
     * Check if the token is expired
     *
     * @return bool
     */
    public function isTokenExpired()
    {
        $refreshToken = $this->getRefreshToken();
        $expiresAt = $this->integration->connection ? $this->integration->connection->token_expires_at : null;

        if ($refreshToken and $expiresAt) {
            return $expiresAt < date('Y-m-d H:i:s');
        }

        return false;
    }

    /**
     * Ping the service API
     *
     * @return bool
     */
    public function ping()
    {
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];

        try {
            // https://developer.mautic.org/#list-available-owners
            $response = $this->client->get($this->config['api_url'] . '/contacts/list/owners', $data);

            if ($response->getStatusCode() == 200 and $result = @json_decode($response->getBody())) {
                return true;
            }

            Log::error('[MAUTIC] Failed to ping.');
        } catch (Exception $e) {
            Log::error('[MAUTIC] Error when pinging. ' . $e->getMessage());
        }

        return false;
    }

    /**
     * Check if integration is ready
     *
     * @return bool
     */
    public function isReady()
    {
        if ($this->client) {
            return true;
        }

        return false;
    }

    /**
     * Return OAuth token for this integration
     *
     * @return string
     */
    public function getToken()
    {
        return $this->integration->getConnectionValue('token');
    }

    /**
     * Return OAuth refresh token for this integration
     *
     * @return string
     */
    public function getRefreshToken()
    {
        return $this->integration->getConnectionValue('refresh_token');
    }

    /**
     * Return token secret for this integration (not needed for MC
     *
     * @return null
     */
    public function getTokenSecret()
    {
        return $this->integration->getConnectionValue('token_secret');
    }

    /**
     * Get all campaigns.
     * Mautic integration have campaigns instead of lists.
     *
     * @return IntegrationListsCollection
     */
    public function getLists()
    {
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];

        try {
            // https://developer.mautic.org/#list-campaigns
            $response = $this->client->get($this->config['api_url'] . '/campaigns', $data);

            if ($response->getStatusCode() == 200 and $result = @json_decode($response->getBody()) and isset($result->campaigns)) {
                $lists = new IntegrationListsCollection;

                foreach ($result->campaigns as $campaign) {
                    if (!$campaign->isPublished)
                        continue;

                    $list = $this->listObject([
                        'id' => $campaign->id,
                        'name' => $campaign->name,
                        'description' => $campaign->description,
                        'form_id' => $campaign->forms ? $campaign->forms[0]->id : "0",
                    ]);

                    $lists->push($list);
                }

                return $lists;
            }

            Log::error('[MAUTIC] Failed to get campaigns.');
        } catch (Exception $e) {
            Log::error('[MAUTIC] Error when fetching campaigns from service. ' . $e->getMessage());
        }

        return new IntegrationListsCollection;
    }

    /**
     * Get Campaign.
     * Mautic integration have campaign instead of list.
     *
     * @param  string  $listId
     * @return IntegrationList
     */
    public function getList($campaignId)
    {
        if ($campaignId) {
            $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];

            try {
                // https://developer.mautic.org/#get-campaign
                $response = $this->client->get($this->config['api_url'] . '/campaigns/' . $campaignId, $data);

                if ($response->getStatusCode() == 200 and $result = @json_decode($response->getBody()) and isset($result->campaign)) {
                    $list = $this->listObject([
                        'id' => $result->campaign->id,
                        'name' => $result->campaign->name,
                        'description' => $result->campaign->description,
                        'form_id' => $result->campaign->forms ? $result->campaign->forms[0]->id : "0",
                    ]);

                    return $list;
                }

                Log::error("[MAUTIC] Failed to get campaign.");
            } catch (Exception $e) {
                Log::error("[MAUTIC] Error when fetching campaign from service. " . $e->getMessage());
            }
        }
    }

    /**
     * Get campaign form fields.
     *
     * @param  mixed  $campaignId
     * @return IntegrationFieldsCollection
     */
    public function getFields($campaignId = null)
    {
        if (! $campaignId)
            return false;

        $campaign = $this->getList($campaignId);
        $formId = $campaign->toArray()['form_id'];
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];

        try {
            // https://developer.mautic.org/#get-form
            $response = $this->client->get($this->config['api_url'] . '/forms/' . $formId, $data);

            if ($response->getStatusCode() == 200 and $result = @json_decode($response->getBody()) and isset($result->form)) {
                $fields = new IntegrationFieldsCollection();

                foreach ($result->form->fields as $field) {
                    if (in_array($field->alias, [ 'submit' ]))
                        continue;

                    $fields->push(new IntegrationField([
                        'id'         => $field->alias,
                        'name'       => $field->alias,
                        'label'      => $field->label,
                        'optin_id'   => (string) $field->id,
                        'type'       => $field->type === "email" ? $field->type : 'text',
                        'param_type' => $field->type === "email" ? $field->type : 'text',
                        'enabled'    => false,
                        'required'   => $field->isRequired,
                        'order'      => $field->order,
                    ]));
                }

                return $fields;
            }

            Log::error("[MAUTIC] Failed to get form [$formId] data.");
        } catch (Exception $e) {
            Log::error("[MAUTIC] Error when fetching form [$formId] from service. " . $e->getMessage());
        }

        return new IntegrationFieldsCollection();
    }

    /**
     * Get all tags
     *
     * @return IntegrationTagsCollection
     */
    public function getTags()
    {
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];

        try {
            // https://developer.mautic.org/#list-tags
            $response = $this->client->get($this->config['api_url'] . '/tags', $data);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->tags)) {
                $tags = new IntegrationTagsCollection;
                foreach ($result->tags as $tag) {
                    $tags->push(new IntegrationTag([
                        'id'   => $tag->tag,
                        'name' => $tag->tag,
                    ]));
                }

                return $tags;
            }

            Log::error("[MAUTIC] Failed to get tags.");
        } catch (Exception $e) {
            Log::error("[MAUTIC] Error when fetching tags from service. ". $e->getMessage());
        }

        return new IntegrationTagsCollection;
    }

    /**
     * Return a specific tag.
     *
     * @param  mixed  $tagId
     * @return IntegrationTag
     */
    public function getTag($tagId)
    {
        $allTags = $this->getTags();

        foreach ($allTags as $tag) {
            if ($tag->id === $tagId) {
                return $tag;
            }
        }

        return null;
    }

    /**
     * Optin to email service.
     *
     * @param  OptinData   $optinData
     * @param  LeadOptions $leadOptions
     * @return OptinResult
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null)
    {
        // Setup params
        $result = $this->optinErrorResult();
        $email  = $optinData->email();
        $contactData = [];

        $campaignId = $optinData->listId();
        $fields = $this->getFields($campaignId);
        $formData = $optinData->all();

        // Prepared fields that user populated
        if ($fields) {
            foreach ($fields as $field) {
                $value = array_get($formData, $field->id);
                if ($value)
                    $contactData[$field->id] = $value;
            }
        }

        // Prepare GDPR Note
        $gdprNote = $this->getGdprNote($optinData);
        $fieldName = $optinData->get('optin-gdpr-field-note');
        if (! empty($gdprNote) && ! empty($fieldName)) {
            $contactData[$fieldName] = $gdprNote;
        }

        // Prepare GDPR tags
        if (class_exists('\OPDashboard\Services\GeoIp')) {
            $optinData->isFromEu = GeoIp::isFromEu();
        } else {
            $optinData->isFromEu = true;
        }

        $gdpr = $this->processGdprTags($optinData, true);
        $gdpr = array_keys($gdpr);

        // Prepare default tags
        $optinTags = $optinData->tags();
        $realTags = [];
        if (!empty($optinTags)) {
            foreach ($optinTags as $tag) {
                $realTags[] = $tag;
            }
        }

        // Merge GDPR tags and default tags
        $tags = array_merge($realTags, $gdpr);

        // Prepare tags
        if ($tags) {
            $contactData['tags'] = $tags;
        }

        $contact = $this->createContact($contactData);
        if ($contact) {
            $contactAddedToCampaign = $this->addContactToCampaign($contact->id , $campaignId);
            if ($contactAddedToCampaign)
                $result = $this->optinSuccessResult();
        }

        return $result;
    }

    /**
     * Create contact.
     *
     * @param array $contactData
     * @throws OptinException
     * @throws AlreadySubscribedException
     * @return array
     */
    public function createContact($contactData)
    {
        $url = $this->config['api_url'] . '/contacts/new';
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];
        $data['json'] = $contactData;

        try {
            // https://developer.mautic.org/#create-contact
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 201 and $result = @json_decode($response->getBody()) and isset($result->contact)) {
                $contact = $result->contact->fields->all;
                Log::debug("[MAUTIC] User [$contact->email] created.");

                return $contact;
            } else if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->contact)) {
                $this->throwOptinAlreadySubscribedException();
            } else {
                Log::error("[MAUTIC] Failed to create user.");
                $this->throwOptinException();
            }
        } catch (Exception $e) {
            Log::error("[MAUTIC] Error when creating contact. " . $e->getMessage());
            $this->throwOptinException($e);
        }

        return [];
    }

    /**
     * Subscribe contact to Campaign.
     *
     * @param string $contactId
     * @param string $campaignId
     * @throws OptinException
     * @return bool
     */
    public function addContactToCampaign($contactId, $campaignId)
    {
        $data = ['headers' => ['Accept' => 'application/json', 'Authorization' => 'Bearer ' . $this->getToken()]];
        $url = $this->config['api_url'] . '/campaigns/' . $campaignId . '/contact/' . $contactId . '/add';
        try {
            // https://developer.mautic.org/#add-contact-to-a-campaign
            $response = $this->client->post($url, $data);

            if ($response->getStatusCode() === 200 and $result = @json_decode($response->getBody()) and isset($result->success)) {
                Log::error("[MAUTIC] Contact [$contactId] subscribed to [$campaignId].");
                return true;
            }
        } catch (Exception $e) {
            Log::error("[MAUTIC] Error when subscribing contact [$contactId] to campaign [$campaignId]. " . $e->getMessage());
            $this->throwOptinException($e);
        }

        return false;
    }

    /**
     * Client ID from env var
     *
     * @return string
     */
    public function getEnvClientId()
    {
        return getenv('MAUTIC_CLIENT_ID');
    }

    /**
     * Client secret from env var
     *
     * @return string
     */
    public function getEnvClientSecret()
    {
        return getenv('MAUTIC_CLIENT_SECRET');
    }

}
