<?php

namespace OptimizePress\Integrations\Services\Membership;

use OptimizePress\Integrations\Collections\IntegrationFieldsCollection;
use OptimizePress\Integrations\IntegrationConfig;
use OptimizePress\Integrations\IntegrationField;
use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\Services\Contracts\IntegrationHasTagsInterface;
use OptimizePress\Integrations\Services\Exceptions\OptinException;
use OptimizePress\Integrations\Storage\IntegrationEloquentModel;
use OptimizePress\Integrations\Storage\IntegrationWpModel;
use OptimizePress\Support\Collection;
use function OptimizePress\Support\array_get;

class MembershipIntegration
{
    /**
     * @var MembershipProvider
     */
    public $provider;

    /**
     * @var string
     */
    public $type;

    /**
     * The integration service
     *
     * @var MembershipIntegrationInterface
     */
    protected $service;

    /**
     * @var array
     */
    protected $data = [];

    /**
     * @var string
     */
    protected $icon;

    /**
     * Initialize new integration provider
     *
     * @param string $provider
     * @param array  $data
     */
    public function __construct($provider, $data = [])
    {
        $this->provider = \OptimizePress\Integrations\get_membership_provider($provider);
        $this->config   = $this->getConfig($provider);
        $this->setup($data);
    }

    /**
     * Get configuration for a provider
     *
     * @param  string $provider
     * @return mixed
     */
    public function getConfig($provider = null)
    {
        $config = include(__DIR__.'/../../../config/providers.php');

        if (isset($config[$provider])) {
            $this->config = new IntegrationConfig($config[$provider]);

            return $this->config;
        }

        return null;
    }

    /**
     * Get the service used to handle provider integration
     *
     * @return MembershipIntegrationInterface
     */
    public function service(): MembershipIntegrationInterface
    {
        if (! $this->service) {
            $serviceClass = $this->config->service;

            if ($serviceClass) {
                $this->service = new $serviceClass($this);
            }
        }

        return $this->service;
    }

    /**
     * Setup integration data
     *
     * @param  mixed  $data
     * @return MembershipIntegration
     */
    public function setup($data = []): MembershipIntegration
    {
        // Add model if exists
        if (is_a($data, IntegrationEloquentModel::class) or is_a($data, IntegrationWpModel::class)) {
            $this->model = $data;
            $data = $data->toArray();
        }

        // Get service connection data
        $connectionData = isset($data['connection_data']) ? $data['connection_data'] : null;
        if (is_string($connectionData)) {
            $connectionData = @json_decode($connectionData);
        } elseif (is_array($connectionData)) {
            $connectionData = (object) $connectionData;
        } elseif (is_object($connectionData)) {
            $connectionData = (object) $connectionData;
        } else {
            $connectionData = (object) [];
        }

        // Setup the data array
        $this->connection = $connectionData;
        $this->type       = $this->config->type;
        $this->icon       = '/assets/img/membership-integration-'.$this->provider.'.png';
        $this->data = [
            'uid'            => $this->uid,
            'title'          => ($this->model && $this->model->title) ? $this->model->title : $this->config->title,
            'provider_title' => $this->config->title,
            'provider_id'    => $this->config->key,
            'provider'       => &$this->provider,
            'type'           => $this->config->type,
            'authorized'     => (bool) (isset($data['authorized']) ? $data['authorized'] : false),
            'class'          => array_get($data, 'service'),
            'grant_label'    => array_get($data, 'grant_label'),
            'revoke_label'   => array_get($data, 'revoke_label'),
        ];

        return $this;
    }

    /**
     * Return integration provider
     *
     * @return MembershipProvider
     */
    public function provider(): MembershipProvider
    {
        return $this->provider;
    }

    /**
     * Return integration provider ID/key
     *
     * @return string
     */
    public function providerId(): string
    {
        return $this->provider ? $this->provider->key : '';
    }

    /**
     * Check if integration provider is active
     *
     * @return bool
     */
    public function isActive() :bool
    {
        return $this->service()->isActive();
    }

    /**
     * Check if integration provider is connected
     *
     * @return bool
     */
    public function isConnected() :bool
    {
        return $this->isActive();
    }

    /**
     * Return connection value for integration
     *
     * @param  string $key
     * @return string
     */
    public function getConnectionValue($key)
    {
        return isset($this->connection->{$key}) ? $this->connection->{$key} : false;
    }

    /**
     * Set connection value for integration
     *
     * @param  string $key
     * @return string
     */
    public function setConnectionValue($key, $value)
    {
        return $this->connection->{$key} = $value;
    }

    /**
     * Get all levels for the membership integration
     *
     * @return Collection
     */
    public function getLevels(): Collection
    {
        return $this->service()->getLevels();
    }

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function optin(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult
    {
        return $this->service()->optin($optinData, $leadOptions);
    }

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     * @throws OptinException
     */
    public function grantLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult
    {
        return $this->service()->grantLevelAccess($optinData, $leadOptions);
    }

    /**
     * Revoke user access to membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return bool
     */
    public function revokeLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :bool
    {
        return $this->service()->revokeLevelAccess($optinData, $leadOptions);
    }

    /**
     * Get all form fields
     *
     * @return IntegrationFieldsCollection
     */
    public function getFields()
    {
        return $this->service()->getFields();
    }

    /**
     * No lists
     *
     * @return bool
     */
    public function hasLists(): bool
    {
        return false;
    }

    /**
     * All have lists
     *
     * @return bool
     */
    public function hasLevels(): bool
    {
        return true;
    }

    /**
     * It is a membership integration
     *
     * @return bool
     */
    public function isMembership(): bool
    {
        return true;
    }

    /**
     * Get all membership levels
     *
     * @return Collection
     */
    public function getMembershipLevels(): Collection
    {
        return $this->service()->getLevels();
    }

    /**
     * No tags
     *
     * @return bool
     */
    public function hasTags(): bool
    {
        return false;
    }

    /**
     * Return config value for integration
     *
     * @param  string $key
     * @return mixed
     */
    public function config($key = null, $default = null)
    {
        return array_get($this->config, $key, $default);
    }

    /**
     * Return a value
     *
     * @param  string $key
     * @return mixed
     */
    public function __get($key)
    {
        return ($this->$key ?? $this->data[$key] ?? null);
    }

    /**
     * Set a value
     *
     * @param string  $key
     * @param mixed   $value
     */
    public function __set($key, $value)
    {
        $this->{$key} = $value;
    }

    /**
     * Change SL authentication mode
     *
     * @param  bool $auth
     * @return void
     */
    public function setSLAuth($auth)
    {
        $this->slAuth = (bool) $auth;
    }

    /**
     * Check if provider have direct oauth integration.
     *
     * @return bool
     */
    public function isDirectOauth()
    {
        return $this->config->direct_oauth;
    }

    /**
     * Check if the integration has a storage model
     *
     * @return bool
     */
    public function hasModel()
    {
        return $this->model;
    }

    /**
     * Is this an OAuth integration
     *
     * @return bool
     */
    public function isOAuth()
    {
        return false;
    }

    /**
     * Is this an E-Mail integration
     *
     * @return bool
     */
    public function isEmail()
    {
        return false;
    }

    /**
     * Is this a Webhook integration
     *
     * @return bool
     */
    public function isWebhook()
    {
        return false;
    }

    /**
     * Does the integration have tags
     *
     * @return bool
     */
    public function hasGoals()
    {
        return $this->config->has_goals ? $this->config->has_goals : false;
    }

    /**
     * Ping the integration service and see if the connection is ok
     *
     * @param  bool  $store
     * @return bool
     */
    public function ping($store = true)
    {
        return true;
    }

    /**
     * Return the integration as an array
     *
     * @return array
     */
    public function toArray()
    {
        return [
            'uid'         => $this->uid,
            'title'       => $this->model ? $this->model->title : $this->config->title,
            'provider'    => $this->provider->key,
            'type'        => $this->type,
            'config'      => $this->config->toArray(),
            'connection'  => (array) $this->connection,
            'model'       => $this->model ? $this->model->toArray() : null,
            'authorized'  => (bool) $this->data['authorized'],
            'sl_auth'     => (bool) $this->slAuth,
            'connect_url' => $this->connectURL,
            'icon'        => $this->icon,
            'levels'      => isset($this->levels) ? $this->levels : new Collection,
            'lists'       => isset($this->lists) ? $this->lists : new Collection,
            'tags'        => isset($this->tags) ? $this->tags : null,
        ];
    }
}
