<?php

namespace OptimizePress\Integrations\Services\Membership\Services;

use OptimizePress\Integrations\LeadOptions;
use OptimizePress\Integrations\OptinData;
use OptimizePress\Integrations\OptinResult;
use OptimizePress\Integrations\Services\Membership\BaseIntegration;
use OptimizePress\Integrations\Services\Membership\MembershipIntegrationInterface;
use OptimizePress\Integrations\Services\Membership\MembershipLevel;
use OptimizePress\Support\Collection;
use OptimizePress\Support\Log\Log;

class RestrictUserAccessIntegration extends BaseIntegration implements MembershipIntegrationInterface
{
    /**
     * Check if integration is active
     *
     * @return bool
     */
    public function isActive(): bool
    {
        return class_exists(\RUA_App::class);
    }

    /**
     * Get all levels for the membership integration
     *
     * @return Collection
     */
    public function getLevels() :Collection
    {
        $levels = new Collection;
        $posts  = \RUA_App::instance()->get_levels();

        if ($posts) {
            foreach ($posts as $post) {
                $levels->push(new MembershipLevel([
                    'id'    => $post->ID,
                    'name'  => $post->post_title,
                    'title' => $post->post_title,
                ]));
            }
        }

        return $levels;
    }

    /**
     * User gets access to specific membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return OptinResult
     */
    public function grantLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :OptinResult
    {
        // Setup params
        $level  = $optinData->listId();
        $result = $this->optinErrorResult();

        // $level, MemberData $data
        if ($level) {
            Log::debug('[OPCART] [RUA] Granting access to level: ' . $level);

            // Find the level post
            $levelPost = get_post($level);

            if ($levelPost) {
                $password = wp_generate_password(12, false);

                // Try to find the user
                $user = get_user_by('email', $optinData->email());

                // If user is not found we create a new one
                if (! $user) {
                    $userId         = wp_create_user($optinData->email(), $password, $optinData->email());
                    $user           = get_user_by('id', $userId);
                    $user->is_new   = true;
                    $user->password = $password;

                    // Update first/last name if needed
                    if ($user) {
                        if ($optinData->firstName()) wp_update_user(['ID' => $user->ID, 'first_name' => $optinData->firstName()]);
                        if ($optinData->lastName())  wp_update_user(['ID' => $user->ID, 'last_name'  => $optinData->lastName()]);
                    }
                } else {
                    // user existed, let us add data about his previous RUA level
                    $user->previousLevel = get_user_meta($user->ID, '_ca_level', true);
                }

                // Add user to membership level
                if ($user && isset($user->ID)) {
                    // Add capabilities
                    add_user_meta($user->ID, '_ca_level', $level);
                    add_user_meta($user->ID, '_ca_level_' . $level, time());

                    if (! $optinData->fromCart) {
                        $this->sendUserNotification($user, $password);
                    }
                }
                Log::debug('[OPCART] [RUA] Access to level [' . $level . '] granted to user [' . $user->ID . ']');

                $result = $this->optinSuccessResult('Access granted.', ['user' => $user]);
            }
        }

        return $result;
    }

    /**
     * Revoke access for user to membership level
     *
     * @param  OptinData    $optinData
     * @param  LeadOptions  $leadOptions
     * @return bool
     */
    public function revokeLevelAccess(OptinData $optinData, LeadOptions $leadOptions = null) :bool
    {
        // Setup params
        $level = $optinData->listId();

        if ($level) {
            Log::debug('[OPCART] [RUA] Revoking access to level: ' . $level);

            // Find the level post
            $levelPost = get_post($level);

            if ($levelPost) {
                // Try to find the user
                $user = get_user_by('email', $optinData->email());

                // Add user to membership level
                if ($user && isset($user->ID)) {
                    // Update capabilities
                    delete_user_meta($user->ID, '_ca_level',   $level);
                    delete_user_meta($user->ID, '_ca_level_' . $level);
                }
                Log::debug('[OPCART] [RUA] Access to level [' . $level . '] revoked for user [' . $user->ID . ']');

                return true;
            }
        }

        return false;
    }
}
